# coding: utf-8

"""
    University Photo API

     ## Introduction  The Photo API allows access to Photos of members of the University. The Photos exposed by this API are expected to be a good representation of the person they relate to.  The API broadly follows the principles of REST and strives to provide an interface that can be easily consumed by downstream systems.  ### Stability  This release of the Photo API is a `beta` offering: a service we are moving towards live but which requires wider testing with a broader group of users.  ### Versioning  The Photo API is versioned using URL path prefixes in the format: `/v1beta1/photos`. This follows the pattern established by the [GCP API](https://cloud.google.com/apis/design/versioning). Breaking changes will not be made without a change in API major version, however non-breaking changes will be introduced without changes to the version path prefix. All changes will be documented in the project's [CHANGELOG](https://gitlab.developers.cam.ac.uk/uis/devops/iam/photo-api/-/blob/master/CHANGELOG.md)  The available versions of the API are listed at the API's root.  ### Domain  The Photo API serves Photos of members of the University, linking a Photo to an individual through `PhotoIdentifiers` (see below) which pertain to a person record within an external system.  The Photos served by the Photo API are expected to be of sufficient quality to identify the person they represent. Photos which have been reviewed and determined to be sufficient quality are marked as having the status `approved` and exposed through the `approved-photos` endpoints.  ### Data source  The Photos exposed by the Photo API are currently a mirror of card Photos contained within the [Card Database](https://webservices.admin.cam.ac.uk/uc/). With Photos being synced from the Card Database to the Photo API hourly.  In future, the Photo API will be extended to allow Photos to be created and 'approved' using the API.  ## Core entities  ### The `Photo` Entity  The `Photo` entity contains information and metadata about a Photo of a member of the university. The `status` field on a Photo indicates whether this Photo has been reviewed and approved. Only Photos which have the status `approved` are exposed through the `approved-photos` endpoints. Each photo entity has a unique identifier - termed UUID - which is used in the system to refer to the specific photo entity. The UUID is provided in JSON documents as the `id` property.   Each Photo entity contains a link to the image data itself and a thumbnail for this Photo, which is guaranteed to be less than 128x128 pixels. Images are served by redirecting to a temporary URL provided by the service used to host images. These temporary URLs will expire after an hour and should not be retained for long-lasting access to a given Photo.  Each Photo may contain a block of JSON metadata recording information about the Photo. Photos cannot be updated once created (apart from to move their status to `approved`) hence this metadata should not be used to store application state or any other information which is not related to the content of the Photo.  Each Photo will have a set of `PhotoIdentifier` which allow the Photo to be linked to a person record within another system.  ### The `PhotoIdentifier` Entity  The `PhotoIdentifier` entity holds the `value` and `scheme` of a given identifier. The `value` field of a `PhotoIdentifier` is a simple ID string - e.g. `wgd23` or `000001`. The `scheme` field of a `PhotoIdentifier` indicates what system this identifier relates to or was issued by. This allows many identifiers which relate to different systems to be recorded against a single `Photo`.  The supported schemes are: * `v1.person.identifiers.cam.ac.uk`: The CRSID of the person represented by this Photo * `person.v1.student-records.university.identifiers.cam.ac.uk`: The CamSIS identifier (USN) of the person represented by this Photo * `person.v1.human-resources.university.identifiers.cam.ac.uk`: The CHRIS identifier (staff number) of the person represented by this Photo * `photo.v1.legacy-card.university.identifiers.cam.ac.uk`: The id of this Photo within the legacy card system  ## Using the API  ### Auth  To authenticate against the Photo API, an application must be registered within the API Gateway and granted access to the `University Photo` product. Details of how to register an application and grant access to products can be found in the [API Gateway Getting Started Guide](https://developer.api.apps.cam.ac.uk/start-using-an-api).  

    The version of the OpenAPI document: v1beta1
    Contact: devops+photoapi@uis.cam.ac.uk
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Any, Dict, List, Optional
from pydantic import BaseModel, Field, StrictInt, StrictStr, conlist
from identitylib.photo_client.models.status_enum import StatusEnum
from identitylib.photo_client.models.v1_beta1_photo_identifier_summary import V1Beta1PhotoIdentifierSummary

class V1Beta1Photo(BaseModel):
    """
    V1Beta1Serializer is the same as the V1Alpha1 version, except for directly returning the identifiers rather than mapping their schemes back to deprecated forms.
    """
    approved_at: Optional[datetime] = Field(None, alias="approvedAt", description="If non-NULL, when this photo was approved as being a good representation of the person it relates to")
    approved_by: Optional[StrictStr] = Field(None, alias="approvedBy", description="The identifier of the user who approved this photo - may not be known for legacy data")
    content_link: StrictStr = Field(..., alias="contentLink")
    created_at: datetime = Field(..., alias="createdAt", description="When this record was created")
    id: StrictStr = Field(...)
    identifiers: conlist(V1Beta1PhotoIdentifierSummary) = Field(...)
    image_height: Optional[StrictInt] = Field(..., alias="imageHeight", description="The height of the image")
    image_width: Optional[StrictInt] = Field(..., alias="imageWidth", description="The width of the image")
    invalidated_at: Optional[datetime] = Field(None, alias="invalidatedAt", description="If non-NULL, the date & time that the photo was invalidated")
    invalidated_by: Optional[StrictStr] = Field(None, alias="invalidatedBy", description="The identifier of the user who invalidated this photo.")
    metadata: Optional[Dict[str, Any]] = Field(None, description="A freeform block of JSON metadata related to this photo")
    original_link: StrictStr = Field(..., alias="originalLink")
    review_postponed_at: Optional[datetime] = Field(None, alias="reviewPostponedAt", description="If non-NULL, the date & time that the photo was last skipped during review process")
    self_link: StrictStr = Field(..., alias="selfLink")
    status: StatusEnum = Field(...)
    submitted_by: StrictStr = Field(..., alias="submittedBy", description="The identifier of the user who submitted this photo if submitted by a user, else the name of the automated process or system")
    thumbnail_link: StrictStr = Field(..., alias="thumbnailLink")
    updated_at: Optional[datetime] = Field(None, alias="updatedAt", description="When this photo was last updated.")
    updated_by: Optional[StrictStr] = Field(None, alias="updatedBy", description="The identifier of the user who updated this photo.")
    __properties = ["approvedAt", "approvedBy", "contentLink", "createdAt", "id", "identifiers", "imageHeight", "imageWidth", "invalidatedAt", "invalidatedBy", "metadata", "originalLink", "reviewPostponedAt", "selfLink", "status", "submittedBy", "thumbnailLink", "updatedAt", "updatedBy"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1Beta1Photo:
        """Create an instance of V1Beta1Photo from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "content_link",
                            "created_at",
                            "id",
                            "image_height",
                            "image_width",
                            "original_link",
                            "self_link",
                            "thumbnail_link",
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in identifiers (list)
        _items = []
        if self.identifiers:
            for _item in self.identifiers:
                if _item:
                    _items.append(_item.to_dict())
            _dict['identifiers'] = _items
        # set to None if approved_at (nullable) is None
        # and __fields_set__ contains the field
        if self.approved_at is None and "approved_at" in self.__fields_set__:
            _dict['approvedAt'] = None

        # set to None if approved_by (nullable) is None
        # and __fields_set__ contains the field
        if self.approved_by is None and "approved_by" in self.__fields_set__:
            _dict['approvedBy'] = None

        # set to None if image_height (nullable) is None
        # and __fields_set__ contains the field
        if self.image_height is None and "image_height" in self.__fields_set__:
            _dict['imageHeight'] = None

        # set to None if image_width (nullable) is None
        # and __fields_set__ contains the field
        if self.image_width is None and "image_width" in self.__fields_set__:
            _dict['imageWidth'] = None

        # set to None if invalidated_at (nullable) is None
        # and __fields_set__ contains the field
        if self.invalidated_at is None and "invalidated_at" in self.__fields_set__:
            _dict['invalidatedAt'] = None

        # set to None if invalidated_by (nullable) is None
        # and __fields_set__ contains the field
        if self.invalidated_by is None and "invalidated_by" in self.__fields_set__:
            _dict['invalidatedBy'] = None

        # set to None if metadata (nullable) is None
        # and __fields_set__ contains the field
        if self.metadata is None and "metadata" in self.__fields_set__:
            _dict['metadata'] = None

        # set to None if review_postponed_at (nullable) is None
        # and __fields_set__ contains the field
        if self.review_postponed_at is None and "review_postponed_at" in self.__fields_set__:
            _dict['reviewPostponedAt'] = None

        # set to None if updated_at (nullable) is None
        # and __fields_set__ contains the field
        if self.updated_at is None and "updated_at" in self.__fields_set__:
            _dict['updatedAt'] = None

        # set to None if updated_by (nullable) is None
        # and __fields_set__ contains the field
        if self.updated_by is None and "updated_by" in self.__fields_set__:
            _dict['updatedBy'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1Beta1Photo:
        """Create an instance of V1Beta1Photo from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1Beta1Photo.parse_obj(obj)

        _obj = V1Beta1Photo.parse_obj({
            "approved_at": obj.get("approvedAt"),
            "approved_by": obj.get("approvedBy"),
            "content_link": obj.get("contentLink"),
            "created_at": obj.get("createdAt"),
            "id": obj.get("id"),
            "identifiers": [V1Beta1PhotoIdentifierSummary.from_dict(_item) for _item in obj.get("identifiers")] if obj.get("identifiers") is not None else None,
            "image_height": obj.get("imageHeight"),
            "image_width": obj.get("imageWidth"),
            "invalidated_at": obj.get("invalidatedAt"),
            "invalidated_by": obj.get("invalidatedBy"),
            "metadata": obj.get("metadata"),
            "original_link": obj.get("originalLink"),
            "review_postponed_at": obj.get("reviewPostponedAt"),
            "self_link": obj.get("selfLink"),
            "status": obj.get("status"),
            "submitted_by": obj.get("submittedBy"),
            "thumbnail_link": obj.get("thumbnailLink"),
            "updated_at": obj.get("updatedAt"),
            "updated_by": obj.get("updatedBy")
        })
        return _obj





# coding: utf-8

"""
    Lookup/Ibis web service API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import re  # noqa: F401
import io
import warnings

from pydantic import validate_arguments, ValidationError
from typing_extensions import Annotated

from pydantic import Field, StrictBool, StrictInt, StrictStr

from typing import Optional

from identitylib.lookup_client.models.group_all_groups200_response import GroupAllGroups200Response
from identitylib.lookup_client.models.group_get_cancelled_members200_response import GroupGetCancelledMembers200Response
from identitylib.lookup_client.models.group_get_group200_response import GroupGetGroup200Response
from identitylib.lookup_client.models.group_search_count200_response import GroupSearchCount200Response

from identitylib.lookup_client.api_client import ApiClient
from identitylib.lookup_client.api_response import ApiResponse
from identitylib.lookup_client.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class GroupApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    def group_all_groups(self, include_cancelled : Annotated[Optional[StrictBool], Field(description="Whether or not to include cancelled groups. By default, only live groups are returned.")] = None, fetch : Annotated[Optional[StrictStr], Field(description="A comma-separated list of any additional attributes or references to fetch.")] = None, **kwargs) -> GroupAllGroups200Response:  # noqa: E501
        """Return a list of all groups.  # noqa: E501

        Return a list of all groups.  By default, only a few basic details about each group are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.group_all_groups(include_cancelled, fetch, async_req=True)
        >>> result = thread.get()

        :param include_cancelled: Whether or not to include cancelled groups. By default, only live groups are returned.
        :type include_cancelled: bool
        :param fetch: A comma-separated list of any additional attributes or references to fetch.
        :type fetch: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GroupAllGroups200Response
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the group_all_groups_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.group_all_groups_with_http_info(include_cancelled, fetch, **kwargs)  # noqa: E501

    @validate_arguments
    def group_all_groups_with_http_info(self, include_cancelled : Annotated[Optional[StrictBool], Field(description="Whether or not to include cancelled groups. By default, only live groups are returned.")] = None, fetch : Annotated[Optional[StrictStr], Field(description="A comma-separated list of any additional attributes or references to fetch.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Return a list of all groups.  # noqa: E501

        Return a list of all groups.  By default, only a few basic details about each group are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.group_all_groups_with_http_info(include_cancelled, fetch, async_req=True)
        >>> result = thread.get()

        :param include_cancelled: Whether or not to include cancelled groups. By default, only live groups are returned.
        :type include_cancelled: bool
        :param fetch: A comma-separated list of any additional attributes or references to fetch.
        :type fetch: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GroupAllGroups200Response, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'include_cancelled',
            'fetch'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method group_all_groups" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('include_cancelled') is not None:  # noqa: E501
            _query_params.append(('includeCancelled', _params['include_cancelled']))

        if _params.get('fetch') is not None:  # noqa: E501
            _query_params.append(('fetch', _params['fetch']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['APIGatewayAuth', 'APIGatewayAuth']  # noqa: E501

        _response_types_map = {
            '200': "GroupAllGroups200Response",
        }

        return self.api_client.call_api(
            '/group/all-groups', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def group_get_cancelled_members(self, groupid : Annotated[StrictStr, Field(..., description="The ID or name of the group.")], fetch : Annotated[Optional[StrictStr], Field(description="A comma-separated list of any additional attributes or references to fetch for each person.")] = None, **kwargs) -> GroupGetCancelledMembers200Response:  # noqa: E501
        """Get all the cancelled members of the specified group, including cancelled members of groups included by the group, and groups included by those groups, and so on.  # noqa: E501

        Get all the cancelled members of the specified group, including cancelled members of groups included by the group, and groups included by those groups, and so on.  By default, only a few basic details about each member are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references of each person.  NOTE: This method returns only cancelled people. It does not include people who were removed from the group. Cancelled people are no longer considered to be current staff, students or accredited visitors, and are no longer regarded as belonging to any groups or institutions. The list returned here reflects their group memberships just before they were cancelled, and so is out-of-date data that should be used with caution.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.group_get_cancelled_members(groupid, fetch, async_req=True)
        >>> result = thread.get()

        :param groupid: The ID or name of the group. (required)
        :type groupid: str
        :param fetch: A comma-separated list of any additional attributes or references to fetch for each person.
        :type fetch: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GroupGetCancelledMembers200Response
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the group_get_cancelled_members_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.group_get_cancelled_members_with_http_info(groupid, fetch, **kwargs)  # noqa: E501

    @validate_arguments
    def group_get_cancelled_members_with_http_info(self, groupid : Annotated[StrictStr, Field(..., description="The ID or name of the group.")], fetch : Annotated[Optional[StrictStr], Field(description="A comma-separated list of any additional attributes or references to fetch for each person.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Get all the cancelled members of the specified group, including cancelled members of groups included by the group, and groups included by those groups, and so on.  # noqa: E501

        Get all the cancelled members of the specified group, including cancelled members of groups included by the group, and groups included by those groups, and so on.  By default, only a few basic details about each member are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references of each person.  NOTE: This method returns only cancelled people. It does not include people who were removed from the group. Cancelled people are no longer considered to be current staff, students or accredited visitors, and are no longer regarded as belonging to any groups or institutions. The list returned here reflects their group memberships just before they were cancelled, and so is out-of-date data that should be used with caution.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.group_get_cancelled_members_with_http_info(groupid, fetch, async_req=True)
        >>> result = thread.get()

        :param groupid: The ID or name of the group. (required)
        :type groupid: str
        :param fetch: A comma-separated list of any additional attributes or references to fetch for each person.
        :type fetch: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GroupGetCancelledMembers200Response, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'groupid',
            'fetch'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method group_get_cancelled_members" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['groupid']:
            _path_params['groupid'] = _params['groupid']


        # process the query parameters
        _query_params = []
        if _params.get('fetch') is not None:  # noqa: E501
            _query_params.append(('fetch', _params['fetch']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['APIGatewayAuth', 'APIGatewayAuth']  # noqa: E501

        _response_types_map = {
            '200': "GroupGetCancelledMembers200Response",
        }

        return self.api_client.call_api(
            '/group/{groupid}/cancelled-members', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def group_get_direct_members(self, groupid : Annotated[StrictStr, Field(..., description="The ID or name of the group.")], fetch : Annotated[Optional[StrictStr], Field(description="A comma-separated list of any additional attributes or references to fetch for each person.")] = None, **kwargs) -> GroupGetCancelledMembers200Response:  # noqa: E501
        """Get the direct members of the specified group, not including members included via groups included by the group.  # noqa: E501

        Get the direct members of the specified group, not including members included via groups included by the group.  By default, only a few basic details about each member are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references of each person.  NOTE: This method will not include cancelled people.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.group_get_direct_members(groupid, fetch, async_req=True)
        >>> result = thread.get()

        :param groupid: The ID or name of the group. (required)
        :type groupid: str
        :param fetch: A comma-separated list of any additional attributes or references to fetch for each person.
        :type fetch: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GroupGetCancelledMembers200Response
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the group_get_direct_members_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.group_get_direct_members_with_http_info(groupid, fetch, **kwargs)  # noqa: E501

    @validate_arguments
    def group_get_direct_members_with_http_info(self, groupid : Annotated[StrictStr, Field(..., description="The ID or name of the group.")], fetch : Annotated[Optional[StrictStr], Field(description="A comma-separated list of any additional attributes or references to fetch for each person.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Get the direct members of the specified group, not including members included via groups included by the group.  # noqa: E501

        Get the direct members of the specified group, not including members included via groups included by the group.  By default, only a few basic details about each member are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references of each person.  NOTE: This method will not include cancelled people.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.group_get_direct_members_with_http_info(groupid, fetch, async_req=True)
        >>> result = thread.get()

        :param groupid: The ID or name of the group. (required)
        :type groupid: str
        :param fetch: A comma-separated list of any additional attributes or references to fetch for each person.
        :type fetch: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GroupGetCancelledMembers200Response, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'groupid',
            'fetch'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method group_get_direct_members" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['groupid']:
            _path_params['groupid'] = _params['groupid']


        # process the query parameters
        _query_params = []
        if _params.get('fetch') is not None:  # noqa: E501
            _query_params.append(('fetch', _params['fetch']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['APIGatewayAuth', 'APIGatewayAuth']  # noqa: E501

        _response_types_map = {
            '200': "GroupGetCancelledMembers200Response",
        }

        return self.api_client.call_api(
            '/group/{groupid}/direct-members', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def group_get_group(self, groupid : Annotated[StrictStr, Field(..., description="The ID or name of the group to fetch. This may be either the numeric ID or the short hyphenated group name (for example `\"100656\"` or `\"cs-editors\"`).")], fetch : Annotated[Optional[StrictStr], Field(description="A comma-separated list of any additional attributes or references to fetch.")] = None, **kwargs) -> GroupGetGroup200Response:  # noqa: E501
        """Get the group with the specified ID or name.  # noqa: E501

        Get the group with the specified ID or name.  By default, only a few basic details about the group are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references of the group.  NOTE: The group returned may be a cancelled group. It is the caller's repsonsibility to check its cancelled flag.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.group_get_group(groupid, fetch, async_req=True)
        >>> result = thread.get()

        :param groupid: The ID or name of the group to fetch. This may be either the numeric ID or the short hyphenated group name (for example `\"100656\"` or `\"cs-editors\"`). (required)
        :type groupid: str
        :param fetch: A comma-separated list of any additional attributes or references to fetch.
        :type fetch: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GroupGetGroup200Response
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the group_get_group_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.group_get_group_with_http_info(groupid, fetch, **kwargs)  # noqa: E501

    @validate_arguments
    def group_get_group_with_http_info(self, groupid : Annotated[StrictStr, Field(..., description="The ID or name of the group to fetch. This may be either the numeric ID or the short hyphenated group name (for example `\"100656\"` or `\"cs-editors\"`).")], fetch : Annotated[Optional[StrictStr], Field(description="A comma-separated list of any additional attributes or references to fetch.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Get the group with the specified ID or name.  # noqa: E501

        Get the group with the specified ID or name.  By default, only a few basic details about the group are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references of the group.  NOTE: The group returned may be a cancelled group. It is the caller's repsonsibility to check its cancelled flag.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.group_get_group_with_http_info(groupid, fetch, async_req=True)
        >>> result = thread.get()

        :param groupid: The ID or name of the group to fetch. This may be either the numeric ID or the short hyphenated group name (for example `\"100656\"` or `\"cs-editors\"`). (required)
        :type groupid: str
        :param fetch: A comma-separated list of any additional attributes or references to fetch.
        :type fetch: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GroupGetGroup200Response, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'groupid',
            'fetch'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method group_get_group" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['groupid']:
            _path_params['groupid'] = _params['groupid']


        # process the query parameters
        _query_params = []
        if _params.get('fetch') is not None:  # noqa: E501
            _query_params.append(('fetch', _params['fetch']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['APIGatewayAuth', 'APIGatewayAuth']  # noqa: E501

        _response_types_map = {
            '200': "GroupGetGroup200Response",
        }

        return self.api_client.call_api(
            '/group/{groupid}', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def group_get_members(self, groupid : Annotated[StrictStr, Field(..., description="The ID or name of the group.")], fetch : Annotated[Optional[StrictStr], Field(description="A comma-separated list of any additional attributes or references to fetch for each person.")] = None, **kwargs) -> GroupGetCancelledMembers200Response:  # noqa: E501
        """Get all the members of the specified group, including members of groups included by the group, and groups included by those groups, and so on.  # noqa: E501

        Get all the members of the specified group, including members of groups included by the group, and groups included by those groups, and so on.  By default, only a few basic details about each member are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references of each person.  NOTE: This method will not include cancelled people.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.group_get_members(groupid, fetch, async_req=True)
        >>> result = thread.get()

        :param groupid: The ID or name of the group. (required)
        :type groupid: str
        :param fetch: A comma-separated list of any additional attributes or references to fetch for each person.
        :type fetch: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GroupGetCancelledMembers200Response
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the group_get_members_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.group_get_members_with_http_info(groupid, fetch, **kwargs)  # noqa: E501

    @validate_arguments
    def group_get_members_with_http_info(self, groupid : Annotated[StrictStr, Field(..., description="The ID or name of the group.")], fetch : Annotated[Optional[StrictStr], Field(description="A comma-separated list of any additional attributes or references to fetch for each person.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Get all the members of the specified group, including members of groups included by the group, and groups included by those groups, and so on.  # noqa: E501

        Get all the members of the specified group, including members of groups included by the group, and groups included by those groups, and so on.  By default, only a few basic details about each member are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references of each person.  NOTE: This method will not include cancelled people.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.group_get_members_with_http_info(groupid, fetch, async_req=True)
        >>> result = thread.get()

        :param groupid: The ID or name of the group. (required)
        :type groupid: str
        :param fetch: A comma-separated list of any additional attributes or references to fetch for each person.
        :type fetch: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GroupGetCancelledMembers200Response, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'groupid',
            'fetch'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method group_get_members" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['groupid']:
            _path_params['groupid'] = _params['groupid']


        # process the query parameters
        _query_params = []
        if _params.get('fetch') is not None:  # noqa: E501
            _query_params.append(('fetch', _params['fetch']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['APIGatewayAuth', 'APIGatewayAuth']  # noqa: E501

        _response_types_map = {
            '200': "GroupGetCancelledMembers200Response",
        }

        return self.api_client.call_api(
            '/group/{groupid}/members', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def group_list_groups(self, groupids : Annotated[StrictStr, Field(..., description="A comma-separated list of group IDs or group names (may be a mix of both).")], fetch : Annotated[Optional[StrictStr], Field(description="A comma-separated list of any additional attributes or references to fetch.")] = None, **kwargs) -> GroupAllGroups200Response:  # noqa: E501
        """Get the groups with the specified IDs or names.  # noqa: E501

        Get the groups with the specified IDs or names.  By default, only a few basic details about each group are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references.  The results are sorted by groupid.  NOTE: The URL path length is limited to around 8000 characters, which limits the number of groups that this method can fetch. Group IDs are currently 6 characters long, and must be comma separated and URL encoded, which limits this method to around 800 groups by ID, but probably fewer by name, depending on the group name lengths.  NOTE: The groups returned may include cancelled groups. It is the caller's repsonsibility to check their cancelled flags.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.group_list_groups(groupids, fetch, async_req=True)
        >>> result = thread.get()

        :param groupids: A comma-separated list of group IDs or group names (may be a mix of both). (required)
        :type groupids: str
        :param fetch: A comma-separated list of any additional attributes or references to fetch.
        :type fetch: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GroupAllGroups200Response
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the group_list_groups_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.group_list_groups_with_http_info(groupids, fetch, **kwargs)  # noqa: E501

    @validate_arguments
    def group_list_groups_with_http_info(self, groupids : Annotated[StrictStr, Field(..., description="A comma-separated list of group IDs or group names (may be a mix of both).")], fetch : Annotated[Optional[StrictStr], Field(description="A comma-separated list of any additional attributes or references to fetch.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Get the groups with the specified IDs or names.  # noqa: E501

        Get the groups with the specified IDs or names.  By default, only a few basic details about each group are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references.  The results are sorted by groupid.  NOTE: The URL path length is limited to around 8000 characters, which limits the number of groups that this method can fetch. Group IDs are currently 6 characters long, and must be comma separated and URL encoded, which limits this method to around 800 groups by ID, but probably fewer by name, depending on the group name lengths.  NOTE: The groups returned may include cancelled groups. It is the caller's repsonsibility to check their cancelled flags.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.group_list_groups_with_http_info(groupids, fetch, async_req=True)
        >>> result = thread.get()

        :param groupids: A comma-separated list of group IDs or group names (may be a mix of both). (required)
        :type groupids: str
        :param fetch: A comma-separated list of any additional attributes or references to fetch.
        :type fetch: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GroupAllGroups200Response, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'groupids',
            'fetch'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method group_list_groups" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('groupids') is not None:  # noqa: E501
            _query_params.append(('groupids', _params['groupids']))

        if _params.get('fetch') is not None:  # noqa: E501
            _query_params.append(('fetch', _params['fetch']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['APIGatewayAuth', 'APIGatewayAuth']  # noqa: E501

        _response_types_map = {
            '200': "GroupAllGroups200Response",
        }

        return self.api_client.call_api(
            '/group/list', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def group_modified_groups(self, min_tx_id : Annotated[StrictInt, Field(..., description="Include modifications made in transactions after (but not including) this one.")], max_tx_id : Annotated[StrictInt, Field(..., description="Include modifications made in transactions up to and including this one.")], groupids : Annotated[Optional[StrictStr], Field(description="Only include groups with IDs or names in this list. By default, all modified groups will be included.")] = None, include_cancelled : Annotated[Optional[StrictBool], Field(description="Include cancelled groups. By default, cancelled groups are excluded.")] = None, membership_changes : Annotated[Optional[StrictBool], Field(description="Include groups whose members have changed. By default, changes to group memberships are not taken into consideration.")] = None, fetch : Annotated[Optional[StrictStr], Field(description="A comma-separated list of any additional attributes or references to fetch.")] = None, **kwargs) -> GroupAllGroups200Response:  # noqa: E501
        """Find all groups modified between the specified pair of transactions.  # noqa: E501

        Find all groups modified between the specified pair of transactions.  The transaction IDs specified should be the IDs from two different requests for the last (most recent) transaction ID, made at different times, that returned different values, indicating that some Lookup data was modified in the period between the two requests. This method then determines which (if any) groups were affected.  By default, only a few basic details about each group are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references.  NOTE: All data returned reflects the latest available data about each group. It is not possible to query for old data, or more detailed information about the specific changes made.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.group_modified_groups(min_tx_id, max_tx_id, groupids, include_cancelled, membership_changes, fetch, async_req=True)
        >>> result = thread.get()

        :param min_tx_id: Include modifications made in transactions after (but not including) this one. (required)
        :type min_tx_id: int
        :param max_tx_id: Include modifications made in transactions up to and including this one. (required)
        :type max_tx_id: int
        :param groupids: Only include groups with IDs or names in this list. By default, all modified groups will be included.
        :type groupids: str
        :param include_cancelled: Include cancelled groups. By default, cancelled groups are excluded.
        :type include_cancelled: bool
        :param membership_changes: Include groups whose members have changed. By default, changes to group memberships are not taken into consideration.
        :type membership_changes: bool
        :param fetch: A comma-separated list of any additional attributes or references to fetch.
        :type fetch: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GroupAllGroups200Response
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the group_modified_groups_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.group_modified_groups_with_http_info(min_tx_id, max_tx_id, groupids, include_cancelled, membership_changes, fetch, **kwargs)  # noqa: E501

    @validate_arguments
    def group_modified_groups_with_http_info(self, min_tx_id : Annotated[StrictInt, Field(..., description="Include modifications made in transactions after (but not including) this one.")], max_tx_id : Annotated[StrictInt, Field(..., description="Include modifications made in transactions up to and including this one.")], groupids : Annotated[Optional[StrictStr], Field(description="Only include groups with IDs or names in this list. By default, all modified groups will be included.")] = None, include_cancelled : Annotated[Optional[StrictBool], Field(description="Include cancelled groups. By default, cancelled groups are excluded.")] = None, membership_changes : Annotated[Optional[StrictBool], Field(description="Include groups whose members have changed. By default, changes to group memberships are not taken into consideration.")] = None, fetch : Annotated[Optional[StrictStr], Field(description="A comma-separated list of any additional attributes or references to fetch.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Find all groups modified between the specified pair of transactions.  # noqa: E501

        Find all groups modified between the specified pair of transactions.  The transaction IDs specified should be the IDs from two different requests for the last (most recent) transaction ID, made at different times, that returned different values, indicating that some Lookup data was modified in the period between the two requests. This method then determines which (if any) groups were affected.  By default, only a few basic details about each group are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references.  NOTE: All data returned reflects the latest available data about each group. It is not possible to query for old data, or more detailed information about the specific changes made.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.group_modified_groups_with_http_info(min_tx_id, max_tx_id, groupids, include_cancelled, membership_changes, fetch, async_req=True)
        >>> result = thread.get()

        :param min_tx_id: Include modifications made in transactions after (but not including) this one. (required)
        :type min_tx_id: int
        :param max_tx_id: Include modifications made in transactions up to and including this one. (required)
        :type max_tx_id: int
        :param groupids: Only include groups with IDs or names in this list. By default, all modified groups will be included.
        :type groupids: str
        :param include_cancelled: Include cancelled groups. By default, cancelled groups are excluded.
        :type include_cancelled: bool
        :param membership_changes: Include groups whose members have changed. By default, changes to group memberships are not taken into consideration.
        :type membership_changes: bool
        :param fetch: A comma-separated list of any additional attributes or references to fetch.
        :type fetch: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GroupAllGroups200Response, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'min_tx_id',
            'max_tx_id',
            'groupids',
            'include_cancelled',
            'membership_changes',
            'fetch'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method group_modified_groups" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('min_tx_id') is not None:  # noqa: E501
            _query_params.append(('minTxId', _params['min_tx_id']))

        if _params.get('max_tx_id') is not None:  # noqa: E501
            _query_params.append(('maxTxId', _params['max_tx_id']))

        if _params.get('groupids') is not None:  # noqa: E501
            _query_params.append(('groupids', _params['groupids']))

        if _params.get('include_cancelled') is not None:  # noqa: E501
            _query_params.append(('includeCancelled', _params['include_cancelled']))

        if _params.get('membership_changes') is not None:  # noqa: E501
            _query_params.append(('membershipChanges', _params['membership_changes']))

        if _params.get('fetch') is not None:  # noqa: E501
            _query_params.append(('fetch', _params['fetch']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['APIGatewayAuth', 'APIGatewayAuth']  # noqa: E501

        _response_types_map = {
            '200': "GroupAllGroups200Response",
        }

        return self.api_client.call_api(
            '/group/modified-groups', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def group_search(self, query : Annotated[StrictStr, Field(..., description="The search string.")], approx_matches : Annotated[Optional[StrictBool], Field(description="Flag to enable more approximate matching in the search, causing more results to be returned. Defaults to `false`. This is ignored for LQL queries.")] = None, include_cancelled : Annotated[Optional[StrictBool], Field(description="Flag to allow cancelled groups to be included. Defaults to `false`.")] = None, offset : Annotated[Optional[StrictInt], Field(description="The number of results to skip at the start of the search. Defaults to 0.")] = None, limit : Annotated[Optional[StrictInt], Field(description="The maximum number of results to return. Defaults to 100.")] = None, order_by : Annotated[Optional[StrictStr], Field(description="The order in which to list the results. This may be `\"groupid\"`, `\"name\"` (the default for non-LQL queries) or `\"title\"`. This is ignored for LQL queries, which always return results in groupid order.")] = None, fetch : Annotated[Optional[StrictStr], Field(description="A comma-separated list of any additional attributes or references to fetch.")] = None, **kwargs) -> GroupAllGroups200Response:  # noqa: E501
        """Search for groups using a free text query string.  # noqa: E501

        Search for groups using a free text query string. This is the same search function that is used in the Lookup web application.  By default, only a few basic details about each group are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references.  NOTE: If the query string starts with the prefix `\"group:\"`, it is treated as an [LQL query](/lql), allowing more advanced searches. An LQL query will ignore the `approxMatches` parameter, but it will respect the value of `includeCancelled`. In addition, an LQL query will ignore the `orderBy` parameter, since LQL queries always return results in ID order.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.group_search(query, approx_matches, include_cancelled, offset, limit, order_by, fetch, async_req=True)
        >>> result = thread.get()

        :param query: The search string. (required)
        :type query: str
        :param approx_matches: Flag to enable more approximate matching in the search, causing more results to be returned. Defaults to `false`. This is ignored for LQL queries.
        :type approx_matches: bool
        :param include_cancelled: Flag to allow cancelled groups to be included. Defaults to `false`.
        :type include_cancelled: bool
        :param offset: The number of results to skip at the start of the search. Defaults to 0.
        :type offset: int
        :param limit: The maximum number of results to return. Defaults to 100.
        :type limit: int
        :param order_by: The order in which to list the results. This may be `\"groupid\"`, `\"name\"` (the default for non-LQL queries) or `\"title\"`. This is ignored for LQL queries, which always return results in groupid order.
        :type order_by: str
        :param fetch: A comma-separated list of any additional attributes or references to fetch.
        :type fetch: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GroupAllGroups200Response
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the group_search_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.group_search_with_http_info(query, approx_matches, include_cancelled, offset, limit, order_by, fetch, **kwargs)  # noqa: E501

    @validate_arguments
    def group_search_with_http_info(self, query : Annotated[StrictStr, Field(..., description="The search string.")], approx_matches : Annotated[Optional[StrictBool], Field(description="Flag to enable more approximate matching in the search, causing more results to be returned. Defaults to `false`. This is ignored for LQL queries.")] = None, include_cancelled : Annotated[Optional[StrictBool], Field(description="Flag to allow cancelled groups to be included. Defaults to `false`.")] = None, offset : Annotated[Optional[StrictInt], Field(description="The number of results to skip at the start of the search. Defaults to 0.")] = None, limit : Annotated[Optional[StrictInt], Field(description="The maximum number of results to return. Defaults to 100.")] = None, order_by : Annotated[Optional[StrictStr], Field(description="The order in which to list the results. This may be `\"groupid\"`, `\"name\"` (the default for non-LQL queries) or `\"title\"`. This is ignored for LQL queries, which always return results in groupid order.")] = None, fetch : Annotated[Optional[StrictStr], Field(description="A comma-separated list of any additional attributes or references to fetch.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Search for groups using a free text query string.  # noqa: E501

        Search for groups using a free text query string. This is the same search function that is used in the Lookup web application.  By default, only a few basic details about each group are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references.  NOTE: If the query string starts with the prefix `\"group:\"`, it is treated as an [LQL query](/lql), allowing more advanced searches. An LQL query will ignore the `approxMatches` parameter, but it will respect the value of `includeCancelled`. In addition, an LQL query will ignore the `orderBy` parameter, since LQL queries always return results in ID order.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.group_search_with_http_info(query, approx_matches, include_cancelled, offset, limit, order_by, fetch, async_req=True)
        >>> result = thread.get()

        :param query: The search string. (required)
        :type query: str
        :param approx_matches: Flag to enable more approximate matching in the search, causing more results to be returned. Defaults to `false`. This is ignored for LQL queries.
        :type approx_matches: bool
        :param include_cancelled: Flag to allow cancelled groups to be included. Defaults to `false`.
        :type include_cancelled: bool
        :param offset: The number of results to skip at the start of the search. Defaults to 0.
        :type offset: int
        :param limit: The maximum number of results to return. Defaults to 100.
        :type limit: int
        :param order_by: The order in which to list the results. This may be `\"groupid\"`, `\"name\"` (the default for non-LQL queries) or `\"title\"`. This is ignored for LQL queries, which always return results in groupid order.
        :type order_by: str
        :param fetch: A comma-separated list of any additional attributes or references to fetch.
        :type fetch: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GroupAllGroups200Response, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'query',
            'approx_matches',
            'include_cancelled',
            'offset',
            'limit',
            'order_by',
            'fetch'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method group_search" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('query') is not None:  # noqa: E501
            _query_params.append(('query', _params['query']))

        if _params.get('approx_matches') is not None:  # noqa: E501
            _query_params.append(('approxMatches', _params['approx_matches']))

        if _params.get('include_cancelled') is not None:  # noqa: E501
            _query_params.append(('includeCancelled', _params['include_cancelled']))

        if _params.get('offset') is not None:  # noqa: E501
            _query_params.append(('offset', _params['offset']))

        if _params.get('limit') is not None:  # noqa: E501
            _query_params.append(('limit', _params['limit']))

        if _params.get('order_by') is not None:  # noqa: E501
            _query_params.append(('orderBy', _params['order_by']))

        if _params.get('fetch') is not None:  # noqa: E501
            _query_params.append(('fetch', _params['fetch']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['APIGatewayAuth', 'APIGatewayAuth']  # noqa: E501

        _response_types_map = {
            '200': "GroupAllGroups200Response",
        }

        return self.api_client.call_api(
            '/group/search', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def group_search_count(self, query : Annotated[StrictStr, Field(..., description="The search string.")], approx_matches : Annotated[Optional[StrictBool], Field(description="Flag to enable more approximate matching in the search, causing more results to be returned. Defaults to `false`. This is ignored for LQL queries.")] = None, include_cancelled : Annotated[Optional[StrictBool], Field(description="Flag to allow cancelled groups to be included. Defaults to `false`.")] = None, **kwargs) -> GroupSearchCount200Response:  # noqa: E501
        """Count the number of groups that would be returned by a search using a free text query string.  # noqa: E501

        Count the number of groups that would be returned by a search using a free text query string.  NOTE: If the query string starts with the prefix `\"group:\"`, it is treated as an [LQL query](/lql), allowing more advanced searches. An LQL query will ignore the `approxMatches` parameter, but it will respect the value of `includeCancelled`.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.group_search_count(query, approx_matches, include_cancelled, async_req=True)
        >>> result = thread.get()

        :param query: The search string. (required)
        :type query: str
        :param approx_matches: Flag to enable more approximate matching in the search, causing more results to be returned. Defaults to `false`. This is ignored for LQL queries.
        :type approx_matches: bool
        :param include_cancelled: Flag to allow cancelled groups to be included. Defaults to `false`.
        :type include_cancelled: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GroupSearchCount200Response
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the group_search_count_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.group_search_count_with_http_info(query, approx_matches, include_cancelled, **kwargs)  # noqa: E501

    @validate_arguments
    def group_search_count_with_http_info(self, query : Annotated[StrictStr, Field(..., description="The search string.")], approx_matches : Annotated[Optional[StrictBool], Field(description="Flag to enable more approximate matching in the search, causing more results to be returned. Defaults to `false`. This is ignored for LQL queries.")] = None, include_cancelled : Annotated[Optional[StrictBool], Field(description="Flag to allow cancelled groups to be included. Defaults to `false`.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Count the number of groups that would be returned by a search using a free text query string.  # noqa: E501

        Count the number of groups that would be returned by a search using a free text query string.  NOTE: If the query string starts with the prefix `\"group:\"`, it is treated as an [LQL query](/lql), allowing more advanced searches. An LQL query will ignore the `approxMatches` parameter, but it will respect the value of `includeCancelled`.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.group_search_count_with_http_info(query, approx_matches, include_cancelled, async_req=True)
        >>> result = thread.get()

        :param query: The search string. (required)
        :type query: str
        :param approx_matches: Flag to enable more approximate matching in the search, causing more results to be returned. Defaults to `false`. This is ignored for LQL queries.
        :type approx_matches: bool
        :param include_cancelled: Flag to allow cancelled groups to be included. Defaults to `false`.
        :type include_cancelled: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GroupSearchCount200Response, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'query',
            'approx_matches',
            'include_cancelled'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method group_search_count" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('query') is not None:  # noqa: E501
            _query_params.append(('query', _params['query']))

        if _params.get('approx_matches') is not None:  # noqa: E501
            _query_params.append(('approxMatches', _params['approx_matches']))

        if _params.get('include_cancelled') is not None:  # noqa: E501
            _query_params.append(('includeCancelled', _params['include_cancelled']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['APIGatewayAuth', 'APIGatewayAuth']  # noqa: E501

        _response_types_map = {
            '200': "GroupSearchCount200Response",
        }

        return self.api_client.call_api(
            '/group/search-count', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

"""
    University Student API

    This API exposes identity information about university students. The purpose of this API is to give access to basic identity information about university students for use by downstream systems, therefore the data exposed is limited to basic information about who a student is and which institutions they are affiliated with.  This API uses the following identifiers to link people to person-records held within different systems:  * `person.v1.student-records.university.identifiers.cam.ac.uk` - the university student (ust) identifier of this person -   also referred to as USN  Additionally this API uses the following identifiers to link people to institutions or entities which a person may have an affiliation to:  * `institution.v1.student-records.university.identifiers.cam.ac.uk` - the university student (ust) institution identifier.     This encompasses colleges, departments and faculties. The     [CamSIS Coding Manual](https://www.camsis.cam.ac.uk/technical/camsis-coding-manual)     lists the available values for this affiliation (under A01 and M01). * `academic-plan.v1.student-records.university.identifiers.cam.ac.uk` - the university student (ust) academic plan     identifier. The     [CamSIS Coding Manual](https://www.camsis.cam.ac.uk/technical/camsis-coding-manual)     lists the available values for this affiliation (under D07).  Both affiliations provide a `status` field which indicate a student's status within an affiliated entity. This `status` field is the short representation of the academic career field held in CamSIS. The [CamSIS Coding Manual](https://www.camsis.cam.ac.uk/technical/camsis-coding-manual) lists the available values for the `status` field (under D05).  This API is currently backed by data from CamSIS, exposing a view of data that is refreshed nightly.  

    The version of the OpenAPI document: v1alpha2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json



from pydantic import BaseModel, Field, StrictStr

class HTTPException(BaseModel):
    """
    A representation of an error, detail about the cause of the error will be provided on the `detail` field.
    """
    detail: StrictStr = Field(...)
    __properties = ["detail"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> HTTPException:
        """Create an instance of HTTPException from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> HTTPException:
        """Create an instance of HTTPException from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return HTTPException.parse_obj(obj)

        _obj = HTTPException.parse_obj({
            "detail": obj.get("detail")
        })
        return _obj





# coding: utf-8

"""
    University Human Resources API

    This API exposes identity information from the University Human Resources system. The purpose of this API is to give access to basic identity information about staff members for use by downstream systems, therefore the data exposed is limited to basic information about who a staff member is and details of any affiliations they have.  This API uses the following identifiers to link people to person-records held within different systems:  * `person.v1.human-resources.university.identifiers.cam.ac.uk` - the university human resources (uhr) identifier of     this person - also known as the staff number of HRN. * `v1.person.identifiers.cam.ac.uk` - the common registration scheme identifier (CRSid) of     this person.  Additionally this API uses the following identifiers to link people to institutions or entities which a person may have an affiliation to:  * `institution.v1.human-resources.university.identifiers.cam.ac.uk` - the university human resources (uhr) institution     identifier. This encompasses departments within the University HR system     (currently CHRIS). * `institution.v1.student-records.university.identifiers.cam.ac.uk` - the student records institution identifier. This     encompasses colleges within the University Student Records system (currently CamSIS).  Affiliations can be used to indicate an institution that a person holds a position within, with the `status` indicating the job role that is held and the scheme of the affiliation being `institution.v1.human-resources.university.identifiers.cam.ac.uk`. Additionally an affiliation can indicate a staff member's line manager with the scheme being `person.v1.human-resources.university.identifiers.cam.ac.uk` and the status being `report`. A staff member can have multiple institution affiliations and multiple report affiliations.  This API is currently backed by data from CHRIS, exposing a view which is updated hourly.  

    The version of the OpenAPI document: v1alpha2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import re  # noqa: F401
import io
import warnings

from pydantic import validate_arguments, ValidationError
from typing_extensions import Annotated

from pydantic import Field, StrictInt, StrictStr

from typing import Optional

from identitylib.hr_client.models.paginated_results_staff_member import PaginatedResultsStaffMember
from identitylib.hr_client.models.staff_member import StaffMember

from identitylib.hr_client.api_client import ApiClient
from identitylib.hr_client.api_response import ApiResponse
from identitylib.hr_client.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class StaffMembersApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    def get_v1alpha2_staff_member_staff_identifier_get(self, identifier : StrictStr, **kwargs) -> StaffMember:  # noqa: E501
        """Get a single staff member  # noqa: E501

        Returns a single staff member based on the identifier provided. The identifier can be provided in the format `<value>@<scheme>`, if the scheme is omitted it shall be assumed that the scheme is `person.v1.human-resources.university.identifiers.cam.ac.uk` (also know as the staff number or HRN).  Currently this endpoint only supports querying by `person.v1.human-resources.university.identifiers.cam.ac.uk` and `v1.person.identifiers.cam.ac.uk` identifiers.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_v1alpha2_staff_member_staff_identifier_get(identifier, async_req=True)
        >>> result = thread.get()

        :param identifier: (required)
        :type identifier: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: StaffMember
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the get_v1alpha2_staff_member_staff_identifier_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.get_v1alpha2_staff_member_staff_identifier_get_with_http_info(identifier, **kwargs)  # noqa: E501

    @validate_arguments
    def get_v1alpha2_staff_member_staff_identifier_get_with_http_info(self, identifier : StrictStr, **kwargs) -> ApiResponse:  # noqa: E501
        """Get a single staff member  # noqa: E501

        Returns a single staff member based on the identifier provided. The identifier can be provided in the format `<value>@<scheme>`, if the scheme is omitted it shall be assumed that the scheme is `person.v1.human-resources.university.identifiers.cam.ac.uk` (also know as the staff number or HRN).  Currently this endpoint only supports querying by `person.v1.human-resources.university.identifiers.cam.ac.uk` and `v1.person.identifiers.cam.ac.uk` identifiers.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_v1alpha2_staff_member_staff_identifier_get_with_http_info(identifier, async_req=True)
        >>> result = thread.get()

        :param identifier: (required)
        :type identifier: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(StaffMember, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'identifier'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_v1alpha2_staff_member_staff_identifier_get" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['identifier']:
            _path_params['identifier'] = _params['identifier']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['ApiGatewayOAuthClientCredentials']  # noqa: E501

        _response_types_map = {
            '200': "StaffMember",
            '400': "HTTPException",
            '404': "HTTPException",
            '422': "HTTPValidationError",
        }

        return self.api_client.call_api(
            '/staff/{identifier}', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def list_v1alpha2_staff_staff_get(self, affiliation : Annotated[Optional[StrictStr], Field(description="An affiliation to filter by, in the format `<value>@<scheme>`. When included only the staff members with a matching affiliation are returned.")] = None, cursor : Annotated[Optional[StrictStr], Field(description="The cursor indicating a unique page of results - this should be auto generated on the `next` and `previous` fields and does not need to be manually added / updated.")] = None, page_size : Annotated[Optional[StrictInt], Field(description="The number of results to return per page. This is limited to 999 results, if a number over 999 is provided the results will be capped at 999. Defaults to 200.")] = None, **kwargs) -> PaginatedResultsStaffMember:  # noqa: E501
        """List staff members  # noqa: E501

        Lists all staff members held within the University Human Resources system, allowing basic filtering by affiliation.  To avoid returning very large responses, this endpoint produces a paged response. Where there are additional results which cannot be returned on a single page, a `next` field will be populated with a url containing the next page of results. These `next` urls should be followed until a page is returned with no `next` field, indicating that there are no further results. The amount of results returned can be customized using the `page_size` query parameter. The maximum amount of results returned is 999.  Staff members can be filtered by affiliation, by providing an `affiliation` query parameter. This affiliation should be provided in the format `<value>@<scheme>`.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_v1alpha2_staff_staff_get(affiliation, cursor, page_size, async_req=True)
        >>> result = thread.get()

        :param affiliation: An affiliation to filter by, in the format `<value>@<scheme>`. When included only the staff members with a matching affiliation are returned.
        :type affiliation: str
        :param cursor: The cursor indicating a unique page of results - this should be auto generated on the `next` and `previous` fields and does not need to be manually added / updated.
        :type cursor: str
        :param page_size: The number of results to return per page. This is limited to 999 results, if a number over 999 is provided the results will be capped at 999. Defaults to 200.
        :type page_size: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PaginatedResultsStaffMember
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the list_v1alpha2_staff_staff_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.list_v1alpha2_staff_staff_get_with_http_info(affiliation, cursor, page_size, **kwargs)  # noqa: E501

    @validate_arguments
    def list_v1alpha2_staff_staff_get_with_http_info(self, affiliation : Annotated[Optional[StrictStr], Field(description="An affiliation to filter by, in the format `<value>@<scheme>`. When included only the staff members with a matching affiliation are returned.")] = None, cursor : Annotated[Optional[StrictStr], Field(description="The cursor indicating a unique page of results - this should be auto generated on the `next` and `previous` fields and does not need to be manually added / updated.")] = None, page_size : Annotated[Optional[StrictInt], Field(description="The number of results to return per page. This is limited to 999 results, if a number over 999 is provided the results will be capped at 999. Defaults to 200.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """List staff members  # noqa: E501

        Lists all staff members held within the University Human Resources system, allowing basic filtering by affiliation.  To avoid returning very large responses, this endpoint produces a paged response. Where there are additional results which cannot be returned on a single page, a `next` field will be populated with a url containing the next page of results. These `next` urls should be followed until a page is returned with no `next` field, indicating that there are no further results. The amount of results returned can be customized using the `page_size` query parameter. The maximum amount of results returned is 999.  Staff members can be filtered by affiliation, by providing an `affiliation` query parameter. This affiliation should be provided in the format `<value>@<scheme>`.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_v1alpha2_staff_staff_get_with_http_info(affiliation, cursor, page_size, async_req=True)
        >>> result = thread.get()

        :param affiliation: An affiliation to filter by, in the format `<value>@<scheme>`. When included only the staff members with a matching affiliation are returned.
        :type affiliation: str
        :param cursor: The cursor indicating a unique page of results - this should be auto generated on the `next` and `previous` fields and does not need to be manually added / updated.
        :type cursor: str
        :param page_size: The number of results to return per page. This is limited to 999 results, if a number over 999 is provided the results will be capped at 999. Defaults to 200.
        :type page_size: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PaginatedResultsStaffMember, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'affiliation',
            'cursor',
            'page_size'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_v1alpha2_staff_staff_get" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('affiliation') is not None:  # noqa: E501
            _query_params.append(('affiliation', _params['affiliation']))

        if _params.get('cursor') is not None:  # noqa: E501
            _query_params.append(('cursor', _params['cursor']))

        if _params.get('page_size') is not None:  # noqa: E501
            _query_params.append(('page_size', _params['page_size']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['ApiGatewayOAuthClientCredentials']  # noqa: E501

        _response_types_map = {
            '200': "PaginatedResultsStaffMember",
            '400': "HTTPException",
            '422': "HTTPValidationError",
        }

        return self.api_client.call_api(
            '/staff', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

"""
    University Human Resources API

    This API exposes identity information from the University Human Resources system. The purpose of this API is to give access to basic identity information about staff members for use by downstream systems, therefore the data exposed is limited to basic information about who a staff member is and details of any affiliations they have.  This API uses the following identifiers to link people to person-records held within different systems:  * `person.v1.human-resources.university.identifiers.cam.ac.uk` - the university human resources (uhr) identifier of     this person - also known as the staff number of HRN. * `v1.person.identifiers.cam.ac.uk` - the common registration scheme identifier (CRSid) of     this person.  Additionally this API uses the following identifiers to link people to institutions or entities which a person may have an affiliation to:  * `institution.v1.human-resources.university.identifiers.cam.ac.uk` - the university human resources (uhr) institution     identifier. This encompasses departments within the University HR system     (currently CHRIS). * `institution.v1.student-records.university.identifiers.cam.ac.uk` - the student records institution identifier. This     encompasses colleges within the University Student Records system (currently CamSIS).  Affiliations can be used to indicate an institution that a person holds a position within, with the `status` indicating the job role that is held and the scheme of the affiliation being `institution.v1.human-resources.university.identifiers.cam.ac.uk`. Additionally an affiliation can indicate a staff member's line manager with the scheme being `person.v1.human-resources.university.identifiers.cam.ac.uk` and the status being `report`. A staff member can have multiple institution affiliations and multiple report affiliations.  This API is currently backed by data from CHRIS, exposing a view which is updated hourly.  

    The version of the OpenAPI document: v1alpha2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from typing import Optional
from pydantic import BaseModel, Field, StrictStr
from identitylib.hr_client.models.affiliation_scheme import AffiliationScheme

class Affiliation(BaseModel):
    """
    A representation of an affiliation - indicating what type of affiliation this is and the entity which the affiliation is related to. Also contains a `status` indicating the position of this staff member in relation to the affiliated entity.
    """
    end: Optional[date] = Field(None, description="The end date of this affiliation. May not be populated if the affiliation is open-ended.")
    scheme: AffiliationScheme = Field(...)
    start: Optional[date] = Field(None, description="The start date of this affiliation. May not be populated if not known.")
    status: StrictStr = Field(..., description="A staff member's status in relation to an affiliated entity. Currently this will be their job title or `report`, indicating that this person reports to the affiliated entity.")
    value: StrictStr = Field(..., description="The value of this affiliation. This will be specific to the human resources system, indicating an entity that this staff member is affiliated with.")
    __properties = ["end", "scheme", "start", "status", "value"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Affiliation:
        """Create an instance of Affiliation from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if end (nullable) is None
        # and __fields_set__ contains the field
        if self.end is None and "end" in self.__fields_set__:
            _dict['end'] = None

        # set to None if start (nullable) is None
        # and __fields_set__ contains the field
        if self.start is None and "start" in self.__fields_set__:
            _dict['start'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Affiliation:
        """Create an instance of Affiliation from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return Affiliation.parse_obj(obj)

        _obj = Affiliation.parse_obj({
            "end": obj.get("end"),
            "scheme": obj.get("scheme"),
            "start": obj.get("start"),
            "status": obj.get("status"),
            "value": obj.get("value")
        })
        return _obj





# coding: utf-8

"""
    Institution Identifiers

    The Insitution Identifiers API provides a mapping between identifiers used in various University databases. The current known database namespaces are:  * `insts.lookup.cam.ac.uk` - [Lookup](https://www.lookup.cam.ac.uk/) instids. * `institution.v1.human-resources.university.identifiers.cam.ac.uk` -   identifiers used in the Human Resources (HR) database. * `institution.v1.student.university.identifiers.cam.ac.uk` - identifiers used   in the Student Information System (SIS) database.  The API provides a single endpoint which can be used to retrieve the current reports as a JSON document. For example, on a machine with [curl](https://curl.se/) installed:  ``` $ curl -o mapping.json https://api.apps.cam.ac.uk/institutions/mapping/v1 ```  The API is useful if you need to reconcile records drawn from multiple databases and determine which institutions are equivalent between them.

    The version of the OpenAPI document: 1.0.0
    Contact: devops@uis.cam.ac.uk
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List
from pydantic import BaseModel, Field, conlist
from identitylib.inst_identifier_client.models.mapping_datum import MappingDatum

class MappingResponse(BaseModel):
    """
    A mapping from Lookup institutions to institutions in other databases.
    """
    institutions: conlist(MappingDatum) = Field(..., description="List of institution mapping data.")
    __properties = ["institutions"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> MappingResponse:
        """Create an instance of MappingResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in institutions (list)
        _items = []
        if self.institutions:
            for _item in self.institutions:
                if _item:
                    _items.append(_item.to_dict())
            _dict['institutions'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> MappingResponse:
        """Create an instance of MappingResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return MappingResponse.parse_obj(obj)

        _obj = MappingResponse.parse_obj({
            "institutions": [MappingDatum.from_dict(_item) for _item in obj.get("institutions")] if obj.get("institutions") is not None else None
        })
        return _obj





# coding: utf-8

"""
    University Photo API

     ## Introduction  The Photo API allows access to Photos of members of the University. The Photos exposed by this API are expected to be a good representation of the person they relate to.  The API broadly follows the principles of REST and strives to provide an interface that can be easily consumed by downstream systems.  ### Stability  This release of the Photo API is a `beta` offering: a service we are moving towards live but which requires wider testing with a broader group of users.  ### Versioning  The Photo API is versioned using URL path prefixes in the format: `/v1beta1/photos`. This follows the pattern established by the [GCP API](https://cloud.google.com/apis/design/versioning). Breaking changes will not be made without a change in API major version, however non-breaking changes will be introduced without changes to the version path prefix. All changes will be documented in the project's [CHANGELOG](https://gitlab.developers.cam.ac.uk/uis/devops/iam/photo-api/-/blob/master/CHANGELOG.md)  The available versions of the API are listed at the API's root.  ### Domain  The Photo API serves Photos of members of the University, linking a Photo to an individual through `PhotoIdentifiers` (see below) which pertain to a person record within an external system.  The Photos served by the Photo API are expected to be of sufficient quality to identify the person they represent. Photos which have been reviewed and determined to be sufficient quality are marked as having the status `approved` and exposed through the `approved-photos` endpoints.  ### Data source  The Photos exposed by the Photo API are currently a mirror of card Photos contained within the [Card Database](https://webservices.admin.cam.ac.uk/uc/). With Photos being synced from the Card Database to the Photo API hourly.  In future, the Photo API will be extended to allow Photos to be created and 'approved' using the API.  ## Core entities  ### The `Photo` Entity  The `Photo` entity contains information and metadata about a Photo of a member of the university. The `status` field on a Photo indicates whether this Photo has been reviewed and approved. Only Photos which have the status `approved` are exposed through the `approved-photos` endpoints. Each photo entity has a unique identifier - termed UUID - which is used in the system to refer to the specific photo entity. The UUID is provided in JSON documents as the `id` property.   Each Photo entity contains a link to the image data itself and a thumbnail for this Photo, which is guaranteed to be less than 128x128 pixels. Images are served by redirecting to a temporary URL provided by the service used to host images. These temporary URLs will expire after an hour and should not be retained for long-lasting access to a given Photo.  Each Photo may contain a block of JSON metadata recording information about the Photo. Photos cannot be updated once created (apart from to move their status to `approved`) hence this metadata should not be used to store application state or any other information which is not related to the content of the Photo.  Each Photo will have a set of `PhotoIdentifier` which allow the Photo to be linked to a person record within another system.  ### The `PhotoIdentifier` Entity  The `PhotoIdentifier` entity holds the `value` and `scheme` of a given identifier. The `value` field of a `PhotoIdentifier` is a simple ID string - e.g. `wgd23` or `000001`. The `scheme` field of a `PhotoIdentifier` indicates what system this identifier relates to or was issued by. This allows many identifiers which relate to different systems to be recorded against a single `Photo`.  The supported schemes are: * `v1.person.identifiers.cam.ac.uk`: The CRSID of the person represented by this Photo * `person.v1.student-records.university.identifiers.cam.ac.uk`: The CamSIS identifier (USN) of the person represented by this Photo * `person.v1.human-resources.university.identifiers.cam.ac.uk`: The CHRIS identifier (staff number) of the person represented by this Photo * `photo.v1.legacy-card.university.identifiers.cam.ac.uk`: The id of this Photo within the legacy card system  ## Using the API  ### Auth  To authenticate against the Photo API, an application must be registered within the API Gateway and granted access to the `University Photo` product. Details of how to register an application and grant access to products can be found in the [API Gateway Getting Started Guide](https://developer.api.apps.cam.ac.uk/start-using-an-api).  

    The version of the OpenAPI document: v1beta1
    Contact: devops+photoapi@uis.cam.ac.uk
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json



from pydantic import BaseModel, Field, StrictStr

class NotFound(BaseModel):
    """
    NotFound
    """
    detail: StrictStr = Field(...)
    __properties = ["detail"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> NotFound:
        """Create an instance of NotFound from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> NotFound:
        """Create an instance of NotFound from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return NotFound.parse_obj(obj)

        _obj = NotFound.parse_obj({
            "detail": obj.get("detail")
        })
        return _obj


